import pandas as pd
from keras.models import Sequential
from keras.layers import Dense
from keras.layers import Dropout
import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
from sklearn.metrics import accuracy_score
from sklearn.metrics import confusion_matrix
from sklearn.preprocessing import StandardScaler
from sklearn.preprocessing import MinMaxScaler
from keras.regularizers import l1, l2

df_under = pd.read_csv("under.csv")
df_over = pd.read_csv("over.csv")
df_test = pd.read_csv("test.csv")
df_org = pd.read_csv("org.csv")

del df_test['Unnamed: 0']
del df_test['Unnamed: 0.1']
del df_org['Unnamed: 0']
del df_under['Unnamed: 0']
del df_over['Unnamed: 0']

pd.set_option("display.max_columns", None)

# Scaling the numeric variables
num_vars = ['age', 'balance', 'day', 'duration', 'campaign', 'pdays', 'previous']

data = [df_org, df_test, df_under, df_over]

for i in data:
  scaler = MinMaxScaler()
  i[num_vars] = scaler.fit_transform(i[num_vars])


def ann(t_data):
    # Load the data
    train_data = t_data
    test_data = df_test

    X_train = train_data.drop("Deposit", axis=1) # Independent variables
    y_train = train_data["Deposit"] # Dependent variable

    X_test = test_data.drop("Deposit", axis=1) # Independent variables
    y_test = test_data["Deposit"] # Dependent variable

    # Build the neural network
    model = Sequential()
    model.add(Dense(32, input_dim=X_train.shape[1], activation="leaky_relu"))
    model.add(Dense(16, activation="leaky_relu"))
    model.add(Dense(1, activation="sigmoid"))

    # Compile the model
    model.compile(loss="binary_crossentropy", optimizer="adam", metrics=["accuracy"])

    # Train the model
    model.fit(x=X_train, y=y_train, epochs=20, batch_size=10, verbose=1, validation_split=.1)

    # Predictions 
    predictions = model.predict(x=X_test, batch_size=10, verbose=0)

    # Binary predictions
    rounded_pred = [np.round(num) for num in predictions]

    # Confusion matrix
    cm = confusion_matrix(y_test, rounded_pred)
    cm = cm/np.sum(cm)
    plt.figure(figsize=(6,6))
    sns.heatmap(cm, annot=True,fmt='0.3%', cmap='Blues')
    plt.ylabel("Actual")
    plt.xlabel("Predicted")


# Original data
ann(df_org)

# Undersampled data
ann(df_under)

# Oversampled data
ann(df_over)